#ifndef __lfn_h
#define __lfn_h

typedef	unsigned char				BYTE;
typedef	unsigned int				WORD;
typedef unsigned long				DWORD;
typedef WORD					QWORD[4];
typedef WORD					FILETIME[4];
typedef WORD					HFILE;
typedef WORD					HFIND;
typedef signed long				SEEKPOS;

//
// lfn errno
//
extern WORD __lfnerrno;

//
// warning level
//
extern BYTE __lfnwarnings;


/* date and time */
                                          //     |   |   |   | 
struct ftime {                            //  5432109876543210         
	int seconds	:5;               //             11111         
	int minutes	:6;               //       111111              
	int hours	:5;               //  11111                                        
#define GET_TIME_SECONDS(t)	((((*((WORD *)&(t))) & 0x001F)     )*2) 
#define GET_TIME_MINUTES(t)	 (((*((WORD *)&(t))) & 0x07E0) >> 5)
#define GET_TIME_HOURS(t)	 (((*((WORD *)&(t))) & 0xF800) >>11)
};  
                                          //     |   |   |   |
struct fdate {                            //  5432109876543210
 	int day		:5;               //             11111
	int month	:4;               //         1111
	int year	:7;               //  1111111
#define GET_DATE_DAY(t)		(((*((WORD *)&(t))) & 0x001F)     ) 
#define GET_DATE_MONTH(t)	(((*((WORD *)&(t))) & 0x01E0) >> 5)
#define GET_DATE_YEAR(t)       ((((*((WORD *)&(t))) & 0xFE00) >> 9)+1980)
};

struct datetime {
	struct	fdate 	d;
	struct	ftime	t;
	WORD		milisecs;
#define DT_GET_TIME_SECONDS(v)	((((*((WORD *)&(v.t))) & 0x001F)     )   *2) 
#define DT_GET_TIME_MINUTES(v)	((((*((WORD *)&(v.t))) & 0x07E0) >> 5)     )
#define DT_GET_TIME_HOURS(v)	((((*((WORD *)&(v.t))) & 0xF800) >>11)     )    
#define DT_GET_DATE_DAY(v)	((((*((WORD *)&(v.d))) & 0x001F)     ) 	   )
#define DT_GET_DATE_MONTH(v)	((((*((WORD *)&(v.d))) & 0x01E0) >> 5)     )
#define DT_GET_DATE_YEAR(v)	((((*((WORD *)&(v.d))) & 0xFE00) >> 9)+1980)
#define DT_GET_MILISECS(v)                    ((v.milisecs)*10)
};

//
void	get_creation_date(
		HFILE handle, 
		struct datetime *dt);

//
void	set_creation_date(
		HFILE handle, 
		struct datetime *dt);

//
void	get_last_access_date(
		HFILE handle, 
		struct datetime *dt);

//
void	set_last_access_date(
		HFILE handle, 
		struct datetime *dt);

//
void	get_last_write_date(
		HFILE handle, 
		struct datetime *dt);

//
void	set_last_write_date(
		HFILE handle, 
		struct datetime *dt);

//
// action
//
#define LONG_CONVERT_TIME_TO_DOS  		0
#define LONG_CONVERT_TIME_FROM_DOS              1
//
void	convert_time(
		struct datetime *dt, 
		FILETIME *fdt, 
		BYTE action);

/*  Directory handling */
//
void	create_directory(
		const char * name);	

//
void	get_default_drive(
		BYTE *drivenum);

//
void	set_default_drive(
		BYTE drivenum);

//
void	get_current_directory(
		char * name, 
		BYTE drivenum);

//
void	set_current_directory(
		const char * name);

//
void	set_short_current_directory(
		const char * name);

//
void	remove_directory(
		const char * name);

//
// format
//
#define LONG_GENERATE_SHORT_NAME_FCB		0x00
#define LONG_GENERATE_SHORT_NAME_DOS		0x01
//
// charset
//
#define CHARSET_WINDOWS_ANSI			0x00
#define CHARSET_CURRENT_OEM                    	0x01
#define CHARSET_UNICODE                        	0x02
#define LONG_GENERATE_SHORT_NAME_CHARSET(source, dest)  (((dest) << 4) |(source))
//
void	generate_short_filename(
		const char *longname, 
		char *shortname,
		BYTE format, 
		BYTE charset);
//
// Action
//
#define LONG_SUBST_CREATE                0x00
#define LONG_SUBST_TERMINATE		 0x01
#define LONG_SUBST_QUERY                 0x02
//
void	subst(
		const char * name, 
		BYTE drivenum, 
		BYTE action);
//
// action
//
#define LONG_TRUENAME_CLASS_CANONICAL		0x00
#define LONG_TRUENAME_CLASS_SHORT		0x01
#define LONG_TRUENAME_CLASS_LONG		0x02
//
// subst
//
#define LONG_TRUENAME_NO_SUBST			0x00
#define LONG_TRUENAME_SUBST			0x80
//
void	truename(
		const char *oldname, 
		char *newname, 
		BYTE action, 
		BYTE subst);

//
//  modes
//
#define OPEN_ACCESS_READONLY        		0x0000
#define OPEN_ACCESS_WRITEONLY           	0x0001
#define OPEN_ACCESS_READWRITE           	0x0002
#define OPEN_ACCESS_RO_NOMODLASTACCESS	 	0x0004
#define	OPEN_SHARE_COMPATIBLE			0x0000
#define	OPEN_SHARE_DENYREADWRITE		0x0010
#define	OPEN_SHARE_DENYWRITE			0x0020
#define OPEN_SHARE_DENYREAD			0x0030
#define	OPEN_SHARE_DENYNONE	                0x0040
#define	OPEN_SHARE_NETWORK_FCB			0x0070
#define	OPEN_FLAGS_NOINHERIT                    0x0080
#define	OPEN_FLAGS_NO_BUFFERING                 0x0100
#define	OPEN_FLAGS_NO_COMPRESS                  0x0200
#define OPEN_FLAGS_ALIAS_HINT                   0x0400
#define	OPEN_FLAGS_NOCRTERR                     0x2000
#define	OPEN_FLAGS_COMMIT                       0x4000
//
// attributes
//
#define FILE_ATTR_NORMAL			  0
#define FILE_ATTR_READONLY              	  1
#define FILE_ATTR_HIDDEN                	  2
#define FILE_ATTR_SYSTEM                 	  4
#define FILE_ATTR_VOLUME_LABEL          	  8
#define FILE_ATTR_EXECUTE_ONLY         		  8	/*NOVELL*/
#define FILE_ATTR_DIRECTORY             	 16
#define FILE_ATTR_ARCHIVE               	 32
#define	FILE_ATTR_SHAREABLE			128	/*NOVELL*/
#define FILE_ATTR_PENDING_DELETED       	128
#define FILE_ATTR_TEMPORARY_FILE		FILE_ATTR_PENDING_DELETED
#define FILE_ATTR_ALL						\
	 FILE_ATTR_NORMAL	|FILE_ATTR_READONLY	        \
	|FILE_ATTR_HIDDEN	|FILE_ATTR_SYSTEM      		\
	|FILE_ATTR_VOLUME_LABEL	|FILE_ATTR_EXECUTE_ONLY         \
	|FILE_ATTR_DIRECTORY 	|FILE_ATTR_ARCHIVE	      	\
	|FILE_ATTR_SHAREABLE	|FILE_ATTR_PENDING_DELETED
//
// action
//
#define	LONG_CREATE_FILE_OPEN		  	1
#define	LONG_CREATE_FILE_TRUNCATE         	2
#define	LONG_CREATE_FILE_CREATE          	16
#define	LONG_CREATE_FILE_CREATE_OPEN 	 (LONG_CREATE_FILE_CREATE | LONG_CREATE_FILE_OPEN)
#define	LONG_CREATE_FILE_CREATE_TRUNCATE (LONG_CREATE_FILE_CREATE | LONG_CREATE_FILE_TRUNCATE)
//
// result
//
#define	LONG_CREATE_FILE_OPENED        		0x0001
#define	LONG_CREATE_FILE_CREATED_OPENED		0x0002
#define	LONG_CREATE_FILE_REPLACED_OPENED	0x0003
//
HFILE	create_file(
		const char * name, 
		WORD mode,
		WORD attributes, 
		WORD * action, 
		WORD alias_hint);

//
HFILE	server_create_file(
		const char * name, 
		WORD mode,
		WORD attributes, 
		WORD * action, 
		WORD alias_hint);

//
void	close_file(
		HFILE handle); 	

//
// wildcardas
//
#define LONG_DELETE_FILE_NO_WILDCARDS		0x0000
#define LONG_DELETE_FILE_WILDCARDS            	0x0001
//
void	delete_file(
		const char * name, 
		WORD wildcard, 
		BYTE searchattrs, 
		BYTE mustmatchattrs);

//
void	rename_file(
		const char *oldname, 
		const char *newname);

//
void	read_file(
		HFILE handle, 
		char *buf, 
		WORD bufsize, 
		WORD *read); 	

//
void	write_file(
		HFILE handle, 
		const char *buf, 
		WORD bufsize, 
		WORD *written); 	 

//
// mode
//
#define	LSEEK_START_FILE_POS			0
#define	LSEEK_CURRENT_FILE_POS                  1
#define	LSEEK_END_FILE_POS                      2	
//
SEEKPOS  lseek(
		HFILE handle, 
		SEEKPOS ofs, 
		BYTE mode);

//
SEEKPOS	get_file_size(
		const char * filename);

//
SEEKPOS	get_compressed_file_size(
		const char * filename);


struct fileinfo {
	DWORD	fattr;		/*file attributes                         */
     FILETIME	creation_time;	/*creation time (0 = unsupported)         */
     FILETIME	access_time;	/*last access time (0 = unsupported)      */
     FILETIME	write_time;	/*last write time                         */
	DWORD	volumeID;	/*volume serial number                    */
	DWORD	hi_filesize;	/*high 32 bits of file size               */
	DWORD	lo_filesize;	/*low 32 bits of file size                */
	DWORD	links;		/*number of links to file                 */
	DWORD	hi_fileID;	/*unique file identifier (high 32 bits)   */
	DWORD	lo_fileID;	/*unique file identifier (low 32 bits)    */
};

//
void	get_file_information(
		HFILE handle, 
		struct fileinfo *finfo);


union fileattrs {
	WORD	attributes;
	DWORD	size;
	struct  datetime dt;
};

//
// action
//
#define LONG_GET_SET_FILE_ATTR_GET_ATTR			0	
#define LONG_GET_SET_FILE_ATTR_SET_ATTR         	1
#define LONG_GET_SET_FILE_ATTR_DRIVE_PHYS_SIZE  	2
#define LONG_GET_SET_FILE_ATTR_SET_WRITE_DATE   	3
#define LONG_GET_SET_FILE_ATTR_GET_WRITE_DATE   	4
#define LONG_GET_SET_FILE_ATTR_SET_ACCESS_DATE  	5
#define LONG_GET_SET_FILE_ATTR_GET_ACCESS_DATE  	6
#define LONG_GET_SET_FILE_ATTR_SET_CREATION_DATE 	7
#define LONG_GET_SET_FILE_ATTR_GET_CREATION_DATE 	8
//
void	get_set_file_attributes(
		const char * name,
		BYTE action, 
		union fileattrs *fa);

/*Find files */
struct finddata {
	DWORD	attributes;	//struct fileattributes attributes_alt;
     FILETIME	creation_datetime;       //0000 0000 0000 0000 0000 0000 0000 0000
     FILETIME	access_datetime;         //date               |time               
     FILETIME	write_datetime;
	DWORD	hi_size;	/*high 32 bits of file size   */
	DWORD	lo_size;	/*low 32 bits of file size    */
 	BYTE	reserved[8];
	BYTE	fullname[260];
	BYTE	shortname[14];
#define GET_FD_DOS_DATE(f)	((*((DWORD *)&(f))) >> 16)      
#define GET_FD_DOS_TIME(f)      ((*((DWORD *)&(f))) & 0xffff)    
};

//
// date type
//
#define LONG_FIND_DATE_FILE			0x0000
#define LONG_FIND_DATE_DOS			0x0001
//
// unicode
//
#define LONG_FIND_UNICODE_NONE			0
#define LONG_FIND_UNICODE_LONG			1
#define LONG_FIND_UNICODE_SHORT			2
//
HFIND	find_first_file(
		BYTE searchattrs, 
		BYTE mustmatchattrs,
		WORD datetype, 
		const char * wildcard,
		struct finddata *fd, 
		BYTE * unicode); 	

//
void	find_next_file(
		HFIND handle, 
		WORD datetype,
		struct finddata *fd, 
		BYTE * unicode);  	

//
void	find_next_file_ext(
		HFIND handle, 
		WORD datetype,
		struct finddata *fd, 
		BYTE * unicode);  	

//
void	find_close(
		HFIND handle);

/* drive */

struct volinfo {
	WORD volflags;
	WORD filename_maxlen;
	WORD pathname_naxlen;
};
//
// volflags 
//
#define	FS_CASE_SENSITIVE   		0x0001
#define FS_CASE_IS_PRESERVED            0x0002
#define FS_UNICODE_ON_DISK              0x0004
#define FS_LFN_APIS                     0x4000
#define	FS_VOLUME_COMPRESSED            0x8000
//
void	get_volume_information(
		const char *rootname,
		char * buffer, 
		WORD bufsize, 
		struct volinfo *v);

//
// action
//
#define LONG_RESET_DRIVE_FLUSH_FSBUFS		0x0000
#define LONG_RESET_DRIVE_FLUSH_FSBUFS_ABD_CACHE 0x0001
#define LONG_RESET_DRIVE_REMOUNT_DRIVESPACE	0x0002
//
void	reset_drive(
		WORD drivenum, 
		WORD action);

struct diskinfo {
	WORD	infolevel;
	DWORD	serialnumber;
	BYTE	volumelabel[11];
	BYTE	fs_type[8];
};
//
// volume label
//
#define FS_FAT12		"FAT12	  "	/*12-bit FAT                            */
#define FS_FAt16		"FAT16	  "	/*16-bit FAT                            */
#define FS_CDROM_HSIERRA	"CDROM	  "	/*High-Sierra CD-ROM filesystem         */
#define FS_CDROM_ISO9660 	"CD001	  "	/*ISO 9660 CD-ROM filesystem            */
#define FS_CDROM_AUDIO 		"CDAUDIO  "	/*audio CD				*/
#define FS_NTFS			"NTFS     "
//
// action
//
void	get_disk_serial_number(
		BYTE drivenum,
		struct diskinfo *dinfo);

//
void	set_disk_serial_number(
		BYTE drivenum,
		struct diskinfo *dinfo);

//
void	error_code(
		const char *name, 
		WORD code);

//
BYTE	check_lfn_redirector(void);

#endif
